# Pydantic
from pydantic import Field
# models
from youtube_pydantic_models.base_model_config import (
    get_base_model_config
)
from youtube_pydantic_models.base_resource import (
    YoutubeBaseResource
)
from youtube_pydantic_models._parts.snippet import (
    ChannelSnippet
)
from youtube_pydantic_models._parts.content_details import (
    ChannelContentDetails
)
from youtube_pydantic_models._parts.statistics import (
    ChannelStatistics
)
from youtube_pydantic_models._parts.topic_details import (
    BaseTopicDetails
)
from youtube_pydantic_models._parts.status import (
    ChannelStatus
)
from youtube_pydantic_models._parts.branding_settings import (
    BaseBrandingSettings
)
from youtube_pydantic_models._parts.audit_details import (
    BaseAuditDetails
)
from youtube_pydantic_models._parts.content_owner_details import (
    BaseContentOwnerDetails
)
from youtube_pydantic_models._subparts.localization import (
    Localization
)


class YoutubeChannelResource(YoutubeBaseResource):
    model_config = get_base_model_config()

    id: str | None = Field(default=None)
    snippet: ChannelSnippet | None = Field(default=None)
    content_details: ChannelContentDetails | None = Field(default=None)
    statistics: ChannelStatistics | None = Field(default=None)
    topic_details: BaseTopicDetails | None = Field(default=None)
    status: ChannelStatus | None = Field(default=None)
    branding_settings: BaseBrandingSettings | None = Field(default=None)
    audit_details: BaseAuditDetails | None = Field(default=None)
    content_owner_details: BaseContentOwnerDetails | None = Field(default=None)
    localizations: dict[str, Localization] | None = Field(default=None)



from youtube_pydantic_models.search_resource import (
    YoutubeSearchResource
)
from tests.test_youtube_resource import TestYoutubeResource


class TestSearchChannelResource(TestYoutubeResource):
    def setUp(self):
        params = self.init_params(
            "search_channel_example_data.json",
            YoutubeSearchResource
        )
        self.json_data = params[0]
        self.model_data = params[1]
    
    def test_equal_kind_part(self):
        self.assert_equal_model_parts("kind")
    
    def test_equal_etag_part(self):
        self.assert_equal_model_parts("etag")

    def test_equal_id_part(self):
        self.assert_equal_model_parts("id")
    
    def test_equal_snippet_part(self):
        self.assert_equal_model_parts("snippet")


class TestSearchPlaylistResource(TestYoutubeResource):
    def setUp(self):
        params = self.init_params(
            "search_playlist_example_data.json",
            YoutubeSearchResource
        )
        self.json_data = params[0]
        self.model_data = params[1]
    
    def test_equal_kind_part(self):
        self.assert_equal_model_parts("kind")
    
    def test_equal_etag_part(self):
        self.assert_equal_model_parts("etag")

    def test_equal_id_part(self):
        self.assert_equal_model_parts("id")
    
    def test_equal_snippet_part(self):
        self.assert_equal_model_parts("snippet")


class TestSearchVideoResource(TestYoutubeResource):
    def setUp(self):
        params = self.init_params(
            "search_video_example_data.json",
            YoutubeSearchResource
        )
        self.json_data = params[0]
        self.model_data = params[1]
    
    def test_equal_kind_part(self):
        self.assert_equal_model_parts("kind")
    
    def test_equal_etag_part(self):
        self.assert_equal_model_parts("etag")

    def test_equal_id_part(self):
        self.assert_equal_model_parts("id")
    
    def test_equal_snippet_part(self):
        self.assert_equal_model_parts("snippet")



import googleapiclient.discovery


class YoutubeClient:
    def __init__(self, api_key: str) -> None:
        API_SERVICE_NAME = "youtube"
        API_VERSION = "v3"
        
        self.client = self.login(
            API_SERVICE_NAME,
            API_VERSION,
            api_key
        )
    
    def login(
        api_service_name: str,
        api_version: str,
        api_key: str
    ):
        try:
            client = googleapiclient.discovery.build(
                api_service_name,
                api_version,
                developerKey = api_key
            )
        except:
            raise Exception("Youtube client not connected")
        return client
    
    def get_client(self):
        return self.client



from youtube_pydantic_models._parts.audit_details import BaseAuditDetails
from youtube_pydantic_models._parts.branding_settings import BaseBrandingSettings
from youtube_pydantic_models._parts.content_details import (
    ChannelContentDetails,
    PlaylistContentDetails,
    VideoContentDetails
)
from youtube_pydantic_models._parts.content_owner_details import BaseContentOwnerDetails
from youtube_pydantic_models._parts.file_details import BaseFileDetails
from youtube_pydantic_models._parts.id import SearchId
from youtube_pydantic_models._parts.live_streaming_details import BaseLiveStreamingDetails
from youtube_pydantic_models._parts.page_info import BasePageInfo
from youtube_pydantic_models._parts.player import (
    BasePlayer,
    VideoPlayer
)
from youtube_pydantic_models._parts.processing_details import BaseProcessingDetails
from youtube_pydantic_models._parts.recording_details import BaseRecordingDetails
from youtube_pydantic_models._parts.snippet import (
    BaseSnippet,
    SearchSnippet,
    ChannelSnippet,
    PlaylistSnippet,
    VideoSnippet
)
from youtube_pydantic_models._parts.statistics import (
    BaseStatistics,
    ChannelStatistics,
    VideoStatistics
)
from youtube_pydantic_models._parts.status import (
    BaseStatus,
    SharedStatus,
    ChannelStatus,
    VideoStatus
)
from youtube_pydantic_models._parts.suggestions import BaseSuggestions
from youtube_pydantic_models._parts.topic_details import (
    BaseTopicDetails,
    VideoTopicDetails
)

from youtube_pydantic_models.base_model_config import get_base_model_config
from youtube_pydantic_models.base_resource import YoutubeBaseResource
from youtube_pydantic_models.channel_resource import YoutubeChannelResource
from youtube_pydantic_models.playlist_resource import YoutubePlaylistResource
from youtube_pydantic_models.video_resource import YoutubeVideoResource
from youtube_pydantic_models.search_resource import YoutubeSearchResource
from youtube_pydantic_models.youtube_client import YoutubeClient
